#!/usr/bin/lua5.1
--[[
-- This script discovers all external USB sound cards, polls for their controls
-- and properties, and exports them to a RAM file in JSON format, where
-- it can be used from other services/apps \(ex. webUI to create dynamically
-- all configuration options, alsa-restore service, etc.\)
-- The idea is to create UDEV rule, that will call that script on USB sound
-- add/remove events, so that this file is kept always up-to-date
-- Ideas for improvements to this script:
-- 1) Fetch the control numid, and add it to the JSON object. 
-- 2) Modify the alsa-usb-detect service, and the webUI to use the numid as
--    UCI entry. This would avoid UCI problems creating entries for controls
--    that have spaces in their names. Ex: "usb_cards.bla_card_name.Mic" will work
--    "but usb_cards.bla_card_name.Auto Gain Control" will fail. Instead, it could
--    be "usb_cards.bla_card_name.4" where "4" is the numid of the Auto Cain Control
--    Numids can be polled using the command "amixer -c1 controls"
--]]

function is_array_empty(arr)
    for _,_ in pairs(arr) do
        return false
    end
    return true
end

function get_json_path()
	local json_fd = io.popen("/usr/bin/uci -q get usb_cards.general.json_file_path | tr -d '\n'")
	local path = json_fd:read("*all")
	json_fd:close()
	if path == nil then 
		path = "/var/run/usb_cards.json" 
	end
	return path
end

function get_control_data(card,control_name)
	--print("Checking control name: "..control_name)
	local data = {}
	local fd = io.popen("amixer -c"..card.." sget \""..control_name.."\"")
    local status = fd:read("*all")
    fd:close()
    
    local limit_sub = string.match(status, "Limits:%s(.*)")
    local limit_low, limit_high
    if limit_sub ~= nil then
    	--print("LIMIT_SUB: "..limit_sub)
    	limit_low = tonumber(string.match(limit_sub, "(%d+)%s-"))
    	limit_high = tonumber(string.match(limit_sub,"-%s(%d?%d?%d)"))
    end
    local volume_pcent = tonumber(string.match(status, "(%d?%d?%d)%%"))
	local volume_db = tonumber(string.match(status, "(-?%d+%.%d+)dB"))
    local on_off = string.match(status, "%[(o[^%]]*)%]")
    

    if limit_low ~= nil then data["limit_low"] = limit_low ; end
    if limit_high ~= nil then data["limit_high"] = limit_high ; end
    if volume_pcent ~= nil then data["volume_pcent"] = volume_pcent ; end
    if volume_db ~= nil then data["volume_db"] = volume_db ; end
    if on_off ~= nil then data["on_off"] = on_off ; end
    
    --get the set of values
    local values = {}
    if limit_low ~=nil and limit_high ~= nil then
    	
		for i=limit_low,limit_high,1 do
			local fd2 = io.popen("amixer -c"..card.." sset \""..control_name.."\" "..i)
			local values_handle = fd2:read("*all")
			fd2:close()
			local volume_db_val = string.match(values_handle, "(-?%d+%.%d+)dB")
			local volume_pcent_val = string.match(values_handle, "(%d?%d?%d)%%")
			--print("\""..i.."\":".."\""..volume_db_val)
			if volume_db_val ~= nil then
				values[tostring(i)] = volume_db_val.."dB"
			else
				if volume_pcent_val ~= nil then
					values[tostring(i)] = volume_pcent_val.."%"
				end
			end
		end
		--restore the initial value
		if volume_db ~= nil and volume_db_val ~=nil then
    		local fd3 = io.popen("amixer -c"..card.." sset \""..control_name.."\" "..volume_db)
    		fd3:close()
    	else
    		if volume_pcent ~= nil and volume_pcent_val ~=nil then
    			local fd3 = io.popen("amixer -c"..card.." sset \""..control_name.."\" "..volume_pcent)
    			fd3:close()
    		end
    	end
	end
    return data, values
end

function get_control_names(card)
	fd = io.popen("amixer -c"..card.." scontrols")
    --local status = fd:read("*line")
    names = {}
    local count=1
    while true do
    	local line = fd:read("*line")
    	if line == nil then break end
    	--print(line)
    	local name = string.match(line,"%'(.+)%'")
    	if name ~= nil then 
    		--print("names["..count.."]="..name)
    		names[count] = name
    		count = count + 1
    	end
 	end
 	fd:close()
	return names
end

function get_audio_cards()
	local fd4 = io.popen("cat /proc/asound/cards | grep \"^ [0-9]\"")
	local cards_list = {}
	local cards_nums = {}
	local count = 1
	while true do
    	local cards_stream = fd4:read("*line")

    	if cards_stream == nil then break end
    	--print("Line: "..cards_stream)
    	local card_num = tonumber(string.match(cards_stream,"%s(%d+)%s"))
    	local card_name = string.match(cards_stream,"% %-% (.+)")
    	--replace all non alphanumeric chars with underscore to avoid UCI conflicts
    	card_name = string.gsub(card_name,"[^%w]","_")
   		cards_list[count] = card_name
   		cards_nums[count] = card_num
    	count = count + 1
    end
    
    fd4:close()
    return cards_list, cards_nums
end

local cards, ids = get_audio_cards()
local card_count = #cards

local json_txt = "{\"usb_cards\": ["
if card_count > 1 then   --skip internal card
	for card_index=2, card_count, 1 do
		local alsa_card_num = ids[card_index]
		local control_names = get_control_names(alsa_card_num)
		local name_count = #control_names
		
		json_txt = json_txt.."{\"id\":\""..alsa_card_num.."\",\"name\":\""..cards[card_index].."\","
		json_txt = json_txt.."\"controls\": [{"
		--print("JSON1: "..json_txt)
		for index=1,name_count,1 do
			--print("Control Name: "..control_names[index])
			json_txt = json_txt.."\"cname\":\""..control_names[index].."\","
			--print("JSON2: "..json_txt)
			local cdata, cvalues = get_control_data(alsa_card_num, control_names[index])
			for key, value in pairs(cdata) do
				--print(key.."="..value)
				json_txt = json_txt.."\""..key.."\":\""..value.."\","
				--print("JSON3: "..json_txt)
			end
	
			-- skip adding values if we do not have them listed 
			local values_is_empty = is_array_empty(cvalues)
			if not values_is_empty then
				-- add this only if we have any values, otherwise
				-- it will create select control with no values on the webUI
				json_txt = json_txt.."\"volume_values\":{"
				for key, value in pairs(cvalues) do
					--print(key.."="..value)
					json_txt = json_txt.."\""..key.."\":\""..value.."\","
				end
			
				--remove the last ","
				json_txt = string.sub(json_txt,1,string.len(json_txt)-1)

				-- close values control
				json_txt = json_txt.."}"
			else
				--remove the last ","
				json_txt = string.sub(json_txt,1,string.len(json_txt)-1)
			end

			-- close this control
			json_txt = json_txt.."}"
	
			if index ~= name_count then
				json_txt = json_txt..",{"
			end
		end
		-- close controls
		json_txt = json_txt.."]"
		-- close current card
		json_txt = json_txt.."}"
		if card_index ~= card_count then
			-- start next card
			json_txt = json_txt..","
		end
	end
end
	
-- close the JSON file
json_txt = json_txt.."]}"

--show it on conc
--print (json_txt)
-- write it to a file
local json_path = get_json_path()
--print("JSON PATH: "..json_path)
local wr_file = io.open(json_path, "w")
wr_file:write(json_txt)
wr_file:close()